#!/bin/bash

SUBNET=$1
NETMASK=$2
ROUTERS=$3
RANGEFROM=$4
RANGETO=$5
TFTP=$6
INTERFACE=$7

usage()
{
	echo "Usage: $0 <subnet> <mask> <gateway> <range ip from> <range ip to> <tftpd server> <dhcp interface>"
	echo "Example: $0 192.168.1.0 255.255.255.0 192.168.1.1 192.168.1.20 192.168.1.29 192.168.1.33 eth0"
	exit 1
}

if [ -z "$7" ] || [ -n "$8" ]
then
	usage
fi

# Configure DHCP

ALLOWBOOTING=`cat /etc/dhcp/dhcpd.conf | grep "^allow booting;"`
if [ -z "${ALLOWBOOTING}" ]
then
	echo "allow booting;" >> /etc/dhcp/dhcpd.conf
fi

ALLOWBOOTP=`cat /etc/dhcp/dhcpd.conf | grep "^allow bootp;"`
if [ -z "${ALLOWBOOTP}" ]
then
	echo "allow bootp;" >> /etc/dhcp/dhcpd.conf
fi

IPXECONF=`cat /etc/dhcp/dhcpd.conf | grep "^option ipxe-no-pxedhcp"`
if [ -z "${IPXECONF}" ]
then
	echo "option ipxe-no-pxedhcp code 176 = unsigned integer 8;" >> /etc/dhcp/dhcpd.conf
fi

CLIENTARCH=`cat /etc/dhcp/dhcpd.conf | grep "^option client-arch"`
if [ -z "${CLIENTARCH}" ]
then
	echo "option client-arch code 93 = unsigned integer 16;" >> /etc/dhcp/dhcpd.conf
fi

cat >> /etc/dhcp/dhcpd.conf <<EOF
## Configuration for the lan
subnet ${SUBNET} netmask ${NETMASK} {
  option routers ${ROUTERS};
  #filename "pxelinux.0";
  if exists user-class and option user-class = "iPXE" {
      filename "http://${TFTP}/script.ipxe";
  } else {
      if option client-arch != 00:00 {
        filename "ipxe.efi";
      } else {
        filename "undionly.kpxe";
      }
  }
  next-server ${TFTP};

  option ipxe-no-pxedhcp 1;

  # Unknown clients get this pool.
  pool {
    max-lease-time 600;
    range dynamic-bootp ${RANGEFROM} ${RANGETO};
    allow unknown-clients;
  }
}
EOF

sed -i -e "s/INTERFACESv4=.*/INTERFACESv4=\"${INTERFACE}\"/g" /etc/default/isc-dhcp-server

/etc/init.d/isc-dhcp-server restart
