#!/bin/bash

DIST=$1
ARCH=$2
TARGET=$3
TFTP=$4
CONFSERVER=$5

usage()
{
	echo "Usage: $0 <dist> <arch> <target> [ <tftpd dir> [ <configuration server ip address> ] ]"
	echo "Example: $0 jessie i386 /home/linux32 /srv/tftp 192.168.100.2"
	echo "Example: $0 stretch amd64 /home/linux64 /srv/tftp"
	echo "Example: $0 buster amd64 /home/linux64 /srv/tftp 192.168.100.2"
	echo "Example: $0 jessie armhf /home/raspberry"
	exit 1
}

if [ -z "$3" ] || [ -n "$6" ]
then
	usage
fi

# RAS_EXTRA_PACKAGES : list of additional packages to install
# RAS_BACKPORT_PACKAGES : list of packages to install from backports

echo "RAS_EXTRA_PACKAGES:"
echo ${RAS_EXTRA_PACKAGES}

echo "RAS_BACKPORT_PACKAGES:"
echo ${RAS_BACKPORT_PACKAGES}

export LANG="es_ES.UTF-8"
KEYBOARD=es
PLYMOUTH_THEME=solar
USERNAME=librework
DNSSERVER=8.8.8.8
PACKAGES="ras live-boot-initramfs-tools live-boot console-data console-setup xserver-xorg xserver-xorg-video-all xserver-xorg-input-all xfonts-base keyboard-configuration apt mawk locales librework-client xinit firmware-linux tightvncserver xtightvncviewer firmware-realtek firmware-bnx2 librsvg2-2 plymouth-themes plymouth initramfs-tools ssh bash-completion nano gnupg xz-utils vim-tiny bzip2 pciutils usbutils less isc-dhcp-client"

if [ -n "${RAS_EXTRA_PACKAGES}" ]
then
	echo "Adding Ras extra packages."
	PACKAGES="${PACKAGES} ${RAS_EXTRA_PACKAGES}"
else
	echo "RAS_EXTRA_PACKAGES is empty. Continuing."
fi

if [ "${DIST}" == "jessie" ]
then
	PACKAGES="${PACKAGES} freerdp-x11 udisks"
fi

if [ "${DIST}" == "stretch" ]
then
	PACKAGES="${PACKAGES} apt-transport-https net-tools systemd-sysv xserver-xorg-legacy xserver-xorg-video-vesa xserver-xorg-video-intel xserver-xorg-input-evdev librework-desktop librework-displaymanager librework-displaymanager-loginscreen iputils-ping dbus-x11 alsa-utils mrpdi-core mrpdi-cli mrpdi-drivers-main mrpdi-drivers-multiclass mrpdi-drivers-iqboard"
fi

if [ "${DIST}" == "buster" ]
then
	PACKAGES="${PACKAGES} apt-transport-https net-tools systemd-sysv xserver-xorg-legacy xserver-xorg-video-vesa xserver-xorg-video-intel xserver-xorg-input-evdev librework-desktop librework-displaymanager librework-displaymanager-loginscreen iputils-ping dbus-x11 alsa-utils freerdp2-x11 ca-certificates"

fi

# Debug
FAST_INSTALL=n
COPY_INITRD=n

TUXUMRASDIR=/usr/share/apps/tuxumras

setInstalled() # setInstalled <package> <target_dir>
{
	cat ${2}/var/lib/dpkg/status | awk -v PACKAGEVAR=${1} '{if ($0 == "Package: " PACKAGEVAR) {MOD=1; print $0} else {if (MOD==1 && $0 == "Status: install ok unpacked") {MOD=0;print "Status: install ok installed"} else { print $0}}}' > ${2}/var/lib/dpkg/status.$1
	mv ${2}/var/lib/dpkg/status.$1 ${2}/var/lib/dpkg/status
}

export LC_MESSAGES=$LANG
export LC_ALL=$LANG
export LANGUAGE=$LANG

START=`date`

if [ "${ARCH}" == "i386" ]
then
	KERNEL="linux-image-686-pae"
	HTYPE="i386"
fi

if [ "${ARCH}" == "armhf" ]
then
	KERNEL=
	HTYPE="arm"
fi

if [ "${ARCH}" == "amd64" ]
then
	KERNEL="linux-image-amd64"
	HTYPE="x86_64"
fi

EXISTS=$(echo ${RAS_BACKPORT_PACKAGES} | grep ${KERNEL})

if [ -z "${EXISTS}" ]
then
	PACKAGES="${PACKAGES} ${KERNEL}"
fi



if [ "${DIST}" == "wheezy" ]
then
	TUXUMREPO=tuxum2012R2
fi

if [ "${DIST}" == "jessie" ]
then
	TUXUMREPO=tuxum2015R2
fi

if [ "${DIST}" == "stretch" ]
then
	TUXUMREPO=tuxum2018R1
fi

if [ "${DIST}" == "buster" ]
then
	TUXUMREPO=tuxum2019R1
fi

cat > /tmp/multistrap.conf <<EOF
[General]
noauth=true
unpack=true
cleanup=true
debootstrap=Debian Tuxum
aptsources=Debian Tuxum

[Debian]
packages=apt
source=http://ftp.debian.org/debian
components=main contrib non-free
keyring=debian-archive-keyring
suite=${DIST}

EOF

if [ "${DIST}" == "buster" ]
then
	cat >> /tmp/multistrap.conf <<EOF
[Tuxum]
packages=${PACKAGES}
source=http://ftp.tuxum.com/${TUXUMREPO}
components=main
suite=${DIST}
EOF
else
	cat >> /tmp/multistrap.conf <<EOF
[Tuxum]
packages=${PACKAGES}
source=http://ftp.tuxum.com/${TUXUMREPO}
components=main
suite=${DIST}
EOF
fi
# Create basic dirs
echo "Creating basic directories..."
mkdir -p ${TARGET}/usr/bin
mkdir -p ${TARGET}/dev/pts
mkdir -p ${TARGET}/proc
mkdir -p ${TARGET}/sys
mkdir -p ${TARGET}/var/lib/config
mkdir -p ${TARGET}/var/lib/systemd
mkdir -p ${TARGET}/var/lib/libnss-librework
echo "Done."

# Create special devices
echo "Creating special devices..."
mknod ${TARGET}/dev/null c 1 3
chmod 666 ${TARGET}/dev/null
mknod ${TARGET}/dev/random c 1 8
mknod ${TARGET}/dev/urandom c 1 9
echo "Done."

# Create random seed
echo "Creating random seed..."
dd if=/dev/urandom of=${TARGET}/var/lib/systemd/random-seed bs=1 count=512
chmod 0600 ${TARGET}/var/lib/systemd/random-seed
echo "Done."

# mount proc, sys and devpts
echo "Mounting proc, sys and devpts..."
mount -t proc proc ${TARGET}/proc
mount -t sysfs none ${TARGET}/sys
mount -t devpts devpts ${TARGET}/dev/pts
echo "Done."

# Configure awk
ln -s mawk ${TARGET}/usr/bin/awk

# Setup configuration server
if [ -n "${CONFSERVER}" ]
then
	mkdir -p ${TARGET}/etc/tuxumras
	echo "CONFIGURATION_SERVER=${CONFSERVER}" > ${TARGET}/etc/tuxumras/tuxumras.conf
fi

# Add apt gpg trusted
mkdir -p ${TARGET}/etc/apt
cp /etc/apt/trusted.gpg ${TARGET}/etc/apt
cp -a /etc/apt/trusted.gpg.d ${TARGET}/etc/apt

# Do chroot environment

multistrap -a ${ARCH} -d ${TARGET} -f /tmp/multistrap.conf

# Clean-up apt
rm -f /tmp/multistrap.conf
rm -f ${TARGET}/etc/apt/sources.list.d/multistrap-*

if [ "${HTYPE}" != ${HOSTTYPE} ]
then
	if [ "${HTYPE}" == "i386" ] && [ "${HOSTTYPE}" == "x86_64" ]
	then
		CHROOT="chroot ${TARGET}"
	else
		cp /usr/bin/qemu-${HTYPE}-static ${TARGET}/usr/bin/
		CHROOT="proot --root-id -q qemu-${HTYPE}-static -r ${TARGET}"
	fi
else
	CHROOT="chroot ${TARGET}"
fi

# Copy tuxumras utilities to target
mkdir -p ${TARGET}/usr/share/apps/tuxumras
cp /usr/share/apps/tuxumras/postinst.sh ${TARGET}/usr/share/apps/tuxumras
cp /usr/share/apps/tuxumras/secondstage.sh ${TARGET}/usr/share/apps/tuxumras

# Generate APT sources list
cat > ${TARGET}/etc/apt/sources.list <<EOF
deb http://ftp.debian.org/debian ${DIST} main contrib non-free
deb http://security.debian.org/ ${DIST}/updates main contrib non-free
deb http://ftp.debian.org/debian ${DIST}-updates main contrib non-free
deb http://deb.debian.org/debian ${DIST}-backports main contrib non-free
EOF

if [ "${DIST}" == "buster" ]
then
	echo -e "deb http://ftp.tuxum.com/${TUXUMREPO} ${DIST} main\n" >> ${TARGET}/etc/apt/sources.list
else
	echo -e "deb http://ftp.tuxum.com/${TUXUMREPO} ${DIST} main\n" >> ${TARGET}/etc/apt/sources.list
fi

# Configure keyboard
mkdir -p ${TARGET}/etc/default
cat > ${TARGET}/etc/default/keyboard <<EOF
XKBMODEL="pc105"
XKBLAYOUT="${KEYBOARD}"
XKBVARIANT=""
XKBOPTIONS=""
BACKSPACE="guess"
EOF
mkdir -p ${TARGET}/etc/console-tools
cp ${TARGET}/usr/share/keymaps/i386/qwerty/${KEYBOARD}.kmap.gz ${TARGET}/etc/console-tools/default.kmap.gz
if [ -e ${TARGET}/etc/console-tools/cached_setup_keyboard.sh ]
then
	sed -i -e "s#loadkeys .*#loadkeys '/etc/console-tools/default.kmap.gz' > '/dev/null'#" ${TARGET}/etc/console-tools/cached_setup_keyboard.sh
fi
for i in ${TARGET}/etc/console-setup/cached_*.kmap.gz
do
	cp ${TARGET}/etc/console-tools/default.kmap.gz $i
done

#Configure locales
if ! [ -e ${TARGET}/etc/locale.gen ]
then
	echo "# es_ES.UTF-8 UTF-8" > ${TARGET}/etc/locale.gen
fi
sed -i "s/# es_ES.UTF-8 UTF-8/es_ES.UTF-8 UTF-8/g" ${TARGET}/etc/locale.gen
${CHROOT} locale-gen
${CHROOT} update-locale LANG=${LANG}

# Install plymouth theme
echo "intel_agp" >> ${TARGET}/etc/initramfs-tools/modules
echo "drm" >> ${TARGET}/etc/initramfs-tools/modules
echo "i915 modeset=1" >> ${TARGET}/etc/initramfs-tools/modules
echo "nouveau modeset=1" >> ${TARGET}/etc/initramfs-tools/modules
echo "radeon modeset=1" >> ${TARGET}/etc/initramfs-tools/modules
${CHROOT} /usr/sbin/plymouth-set-default-theme ${PLYMOUTH_THEME}
if [ "${DIST}" == "stretch" ] || [ "${DIST}" == "buster" ]
then
	cp ${TARGET}/usr/share/librework-displaymanager/logo.png ${TARGET}/usr/share/plymouth/debian-logo.png
else
	cp ${TARGET}/usr/share/apps/tuxumras/logo.png ${TARGET}/usr/share/plymouth/debian-logo.png
fi
setInstalled plymouth ${TARGET}

# Configure X11
mkdir -p ${TARGET}/etc/X11

if [ "${DIST}" == "stretch" ] || [ "${DIST}" == "buster" ]
then
	echo "allowed_users=anybody" >> ${TARGET}/etc/X11/Xwrapper.config
	echo "needs_root_rights=yes" >> ${TARGET}/etc/X11/Xwrapper.config
else
	echo "allowed_users=console" >> ${TARGET}/etc/X11/Xwrapper.config
fi

if ! [ -e ${TARGET}/etc/X11/X ]
then
	ln -s /usr/bin/Xorg ${TARGET}/etc/X11/X
fi

# Comment user creation as now it is created by ras package postinstall script
# chroot ${TARGET} useradd -g staff -G audio,video,cdrom,floppy,plugdev,netdev,saned,scanner,lp,lpadmin,dialout,games,input -m -s /bin/bash ${USERNAME}
# echo "${USERNAME} ALL = NOPASSWD: /sbin/shutdown" > ${TARGET}/etc/sudoers.d/${USERNAME}

# Configure fstab
cat > ${TARGET}/etc/fstab <<EOF
/dev/nfs	/		nfs	tcp,nolock	0	0
proc		/proc           proc	defaults	0	0
none		/tmp		tmpfs	defaults	0	0
none		/media		tmpfs	defaults	0	0
none		/run		tmpfs	defaults	0	0
none		/root		tmpfs	defaults	0	0
none		/var/tmp	tmpfs	defaults	0	0
none		/var/log	tmpfs	defaults	0	0
none		/var/lib/libnss-librework	tmpfs	defaults	0	0
none		/home/librework	tmpfs	user,uid=1000	0	0
EOF

if ! [ "${DIST}" == "stretch" ] || [ "${DIST}" == "buster" ]
then
	echo "none		/home/${USERNAME}	tmpfs	uid=${USERNAME},gid=staff,mode=0700	0	0" >>  ${TARGET}/etc/fstab
fi

rm -f ${TARGET}/etc/mtab
ln -s /proc/mounts ${TARGET}/etc/mtab
if ! [ -e ${TARGET}/dev/fd ]
then
	ln -s /proc/self/fd ${TARGET}/dev/fd
fi

# Configure /etc/machine-id
systemd-machine-id-setup --root=${TARGET}

# Configure basic resolv.conf
echo "nameserver ${DNSSERVER}" > ${TARGET}/etc/resolv.conf

if ! [ -e ${TARGET}/etc/passwd ]
then
	# Configure root account and group
	echo "root:x:0:0:root:/root:/bin/bash" > ${TARGET}/etc/passwd
	echo "root:!:16716:0:99999:7:::" > ${TARGET}/etc/shadow
	echo "root:x:0:" > ${TARGET}/etc/group
fi

if [ "${FAST_INSTALL}" == "y" ]
then
# Configure groups
	cat >> ${TARGET}/etc/group <<EOF
daemon:x:1:
bin:x:2:
sys:x:3:
adm:x:4:
tty:x:5:
disk:x:6:${USERNAME}
lp:x:7:${USERNAME}
mail:x:8:
news:x:9:
uucp:x:10:
man:x:12:
proxy:x:13:
kmem:x:15:
dialout:x:20:
fax:x:21:
voice:x:22:
cdrom:x:24:${USERNAME}
floppy:x:25:${USERNAME}
tape:x:26:
sudo:x:27:
audio:x:29:${USERNAME}
dip:x:30:
www-data:x:33:
backup:x:34:
operator:x:37:
list:x:38:
irc:x:39:
src:x:40:
gnats:x:41:
shadow:x:42:
utmp:x:43:
video:x:44:${USERNAME}
sasl:x:45:
plugdev:x:46:${USERNAME}
staff:x:50:
EOF

fi

# Do some postinstall

mkdir -p ${TARGET}/media

# Configure dash
if [ -e ${TARGET}/var/lib/dpkg/info/dash.preinst ]
then
	${CHROOT} bash /var/lib/dpkg/info/dash.preinst 
fi

# Configure udev
${CHROOT} udevadm hwdb --update
setInstalled udev ${TARGET}

# Configure some packages needed for all
for i in mawk base-passwd libpam-runtime adduser debconf dpkg systemd initscripts keyboard-configuration ras
do
	echo "Setting up $i..."
	${CHROOT} /usr/share/apps/tuxumras/postinst.sh $i
	setInstalled $i ${TARGET}
	echo "Done."
done

if ! [ "${FAST_INSTALL}" == "y" ]
then
	${CHROOT} /usr/share/apps/tuxumras/secondstage.sh
	# Stop dbus daemon on chroot
	${CHROOT} pkill dbus-daemon
fi

# Install selected backports
if [ -n "${RAS_BACKPORT_PACKAGES}" ]
then
	${CHROOT} apt update
	${CHROOT} su - -c "apt -y -t ${DIST}-backports install ${RAS_BACKPORT_PACKAGES}"
fi

# Generate initrd
sed -i -e "s/MODULES=most/MODULES=netboot/g" -e "s/KEYMAP=n/KEYMAP=y/g" ${TARGET}/etc/initramfs-tools/initramfs.conf
for i in ${TARGET}/boot/vmlinuz-*
do
	VERSION=`echo $i | awk -F 'vmlinuz-' '{print $2}'`
	if [ "${COPY_INITRD}" == "y" ]
	then
		${CHROOT} depmod -ae ${VERSION}
		if [ -n "${TFTP}" ]
		then
			cp ${TFTP}/initrd*${VERSION} ${TARGET}/boot
		fi
	else
		if [ -e ${TARGET}/boot/initrd.img-${VERSION} ]
		then
			${CHROOT} update-initramfs -k ${VERSION} -u
		else
			${CHROOT} update-initramfs -k ${VERSION} -c
		fi
		if [ -n "${TFTP}" ]
		then
			cp ${TARGET}/boot/initrd* ${TFTP}
			cp ${TARGET}/boot/vmlinuz* ${TFTP}
		fi
	fi
done

# Umount chroot virtual file systems
umount ${TARGET}/proc
umount ${TARGET}/sys
umount ${TARGET}/dev/pts

# Finish
echo "Done."

END=`date`

echo "Started on ${START}"
echo "Finished on ${END}"
