#!/bin/bash

ARCH=$1
TARGET=$2
NAME=$3
TFTP=$4
NETIFACE=$5

TUXUMRASDIR=/usr/share/apps/tuxumras

usage()
{
	echo "Usage: $0 <arch> <target> <name> <tftpd dir> <network interface>"
	echo "Example: $0 i386 /home/linux32 Linux32 /srv/tftp eth0"
	echo "Example: $0 amd64 /home/linux64 Linux64 /srv/tftp enp2s0f0"
	echo "Example: $0 armhf /home/raspberry Raspberry /srv/tftp eth0"
	exit 1
}

if [ -z "$5" ] || [ -n "$6" ]
then
	usage
fi

START=`date`

# Configure TFTPD

if ! [ "${ARCH}" == "armhf" ]
then
	mkdir -p ${TFTP}

	for i in /usr/lib/syslinux/pxelinux.0 /usr/lib/PXELINUX/pxelinux.0 /usr/lib/syslinux/modules/bios/ldlinux.c32 /usr/lib/ipxe/undionly.kpxe /boot/ipxe.efi
	do
		if [ -e $i ]
		then
			cp $i ${TFTP}
		fi
	done

	mkdir -p ${TFTP}/config
	mkdir -p ${TFTP}/pxelinux.cfg

	if [ -e ${TFTP}/config/${NAME} ]
	then
		rm -f ${TFTP}/config/${NAME}
	fi

	${TUXUMRASDIR}/update_install.sh ${TARGET} ${NAME} ${TFTP} ${NETIFACE}

	rm -f ${TFTP}/pxelinux.cfg/default
	ln -s ../config/${NAME} ${TFTP}/pxelinux.cfg/default
fi

# Configure NFS

EXISTS=`cat /etc/exports | grep -w "${TARGET}" | grep "*(ro,sync,no_root_squash,no_subtree_check)"`

if [ -z "${EXISTS}" ]
then
	echo -e "${TARGET}\t*(ro,sync,no_root_squash,no_subtree_check)" >> /etc/exports
fi

/etc/init.d/nfs-kernel-server restart

# Restart TFTP
if ! [ "${ARCH}" == "armhf" ]
then
	service tftpd-hpa restart || service atftpd restart || service tftpd restart
fi

# Finish
echo "Done."

END=`date`

echo "Started on ${START}"
echo "Finished on ${END}"
