#!/bin/bash

TARGET=$1
CONFIG=$2
TFTP=$3
NETIFACE=$4

APACHE=/etc/apache2

usage()
{
	echo "Usage: $0 <target> <name> <tftpd dir> <network interface>"
	echo "Example: $0 /home/linux32 Linux32 /srv/tftp eth0"
	exit 1
}

if [ -z "$4" ] || [ -n "$5" ]
then
	usage
fi

KERNEL=`ls ${TARGET}/boot/vmlinuz-*`
INITRD=`ls ${TARGET}/boot/initrd*`

if [ -n "${KERNEL}" ]
then
	for i in ${KERNEL}
	do
		KERNELFILE=`basename ${i}`
		cp ${i} ${TFTP}/${KERNELFILE}
	done
fi

if [ -n "${INITRD}" ]
then
	for i in ${INITRD}
	do
		INITRDFILE=`basename ${i}`
		INITRDCONFIG="initrd=${INITRDFILE}"
		cp ${i} ${TFTP}/${INITRDFILE}
	done
fi

DEBIAN_VERSION=`cat /etc/debian_version | awk -F '.' '{print $1}'`

if (( DEBIAN_VERSION > 9 ))
then
	IPADDRESS=`ifconfig ${NETIFACE} | grep -w "inet" | awk '{print $2}'`
else
	IPADDRESS=`ifconfig ${NETIFACE} | grep "inet addr:" | awk '{print $2}' | awk -F ':' '{print $2}'`
fi

cat > ${TFTP}/config/${CONFIG} <<EOF
	IPAPPEND 2
	SERIAL 0 115200                 # show on serial console

	PROMPT 0                        # show boot: prompt
	TIMEOUT 0                       # boot default after 0 seconds
	DEFAULT linux

	label linux
	kernel ${KERNELFILE}
	append root=/dev/nfs ${INITRDCONFIG} ip=dhcp nfsroot=${IPADDRESS}:${TARGET} splash=silent quiet ro panic=30
EOF

cat > ${TFTP}/script.ipxe <<EOF
#!ipxe
  kernel http://${IPADDRESS}/${KERNELFILE} root=/dev/nfs ip=dhcp nfsroot=${IPADDRESS}:${TARGET},nolock,rsize=4096,wsize=4096 splash=silent quiet ro panic=30 initrd=${INITRDFILE}
  initrd http://${IPADDRESS}/${INITRDFILE}
  boot
EOF

cat > ${APACHE}/sites-available/ipxe.conf <<EOF
<VirtualHost *:80>
        ServerAdmin webmaster@localhost
        DocumentRoot /srv/tftp

        <Directory />
                Options FollowSymLinks
                AllowOverride None
                Require all granted
        </Directory>
        #LogLevel info ssl:warn

        ErrorLog ${APACHE_LOG_DIR}/error.log
        CustomLog ${APACHE_LOG_DIR}/access.log combined

        # For most configuration files from conf-available/, which are
        # enabled or disabled at a global level, it is possible to
        # include a line for only one particular virtual host. For example the
        # following line enables the CGI configuration for this host only
        # after it has been globally disabled with "a2disconf".
        #Include conf-available/serve-cgi-bin.conf
</VirtualHost>

# vim: syntax=apache ts=4 sw=4 sts=4 sr noet
EOF

a2dissite 000-default || true 2>&1 >/dev/null
a2ensite ipxe 2>&1 >/dev/null

service apache2 restart
